/* 
 * 
 * Home Health Grouper Software
 * 
 * Center for Medicare and Medicaid Services (CMS)
 * 
 * All code is provided as is.
 * 
 */
package gov.cms.hh.grouper;

import gov.cms.hh.claim.ClaimAdapter;
import gov.cms.hh.claim.ClaimAdapterIF;
import gov.cms.hh.claim.ClaimIF;
import gov.cms.hh.claim.HippsCode;
import gov.cms.hh.claim.ReturnCode;
import gov.cms.hh.data.exchange.ClaimContainer;
import gov.cms.hh.data.exchange.ClaimContainerIF;
import gov.cms.hh.data.exchange.HippsContainer;
import gov.cms.hh.data.exchange.HippsContainerIF;
import gov.cms.hh.data.exchange.ReturnCodeContainer;
import gov.cms.hh.data.exchange.ValidityFlagContainer;
import gov.cms.hh.data.files.FileProperties;
import gov.cms.hh.data.files.TableNames_EN;
import gov.cms.hh.data.loader.PositionAccessorIF;
import gov.cms.hh.data.meta.MetaData;
import static gov.cms.hh.data.meta.MetaData.FORMAT_DATE;
import gov.cms.hh.data.meta.enumer.ActionFlag_EN;
import gov.cms.hh.data.meta.enumer.Claim_EN;
import gov.cms.hh.logic.processor.HippsProcessor;
import gov.cms.hh.logic.validation.EditIF;
import gov.cms.hh.reference.objects.ClaimObject;
import gov.cms.hh.reference.objects.GrouperVersionObject;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author CMS
 */
public abstract class AbstractGrouper implements GrouperIF {

    /**
     * @return the linked
     */
    @Override
    public boolean isLinked() {
        return linked;
    }

    @Override
    public void setLinked() {
        this.linked = true;
    }

    @Override
    public void setUnlinked() {
        this.linked = false;
    }
    
    /**
     * @return the nextVersion
     */
    @Override
    public GrouperIF getNextVersion() {
        return nextVersion;
    }

    /**
     * @param nextVersion the nextVersion to set
     */
    @Override
    public void setNextVersion(GrouperIF nextVersion) {
        this.nextVersion = nextVersion;
    }

    private static final Logger LOG = Logger.getLogger(AbstractGrouper.class.getName());

    private DataManagerIF dataManager;
    private ClaimManagerIF claimManager;
    
    private boolean linked;
    private GrouperIF nextVersion;

    /**
     *
     * @return
     */
    @Override
    public List<Integer> getClaimLayout() {
        return getClaimManager().getClaimLayout();
    }

    /**
     *
     * @return
     */
    @Override
    public List<Integer> getClaimLayoutInputOnly() {
        return getClaimManager().getClaimLayoutInputOnly();
    }

    /**
     * @return the startDate
     */
    @Override
    public Date getStartDate() {
        return new Date(startDate.getTime());
    }

    /**
     * @param startDate the startDate to set
     */
    public void setStartDate(Date startDate) {
        this.startDate = new Date(startDate.getTime());
    }

    /**
     * @return the endDate
     */
    @Override
    public Date getEndDate() {
        return new Date(endDate.getTime());
    }

    /**
     * @param endDate the endDate to set
     */
    public void setEndDate(Date endDate) {
        this.endDate = new Date(endDate.getTime());
    }

    /**
     *
     * @param id
     * @param name
     * @param description
     * @param startDate
     * @param endDate
     * @param active
     * @param defaultt
     * @param loadDescription
     * @throws ParseException
     */
    public AbstractGrouper(int id, String name, String description,
            String startDate, String endDate, boolean active,
            boolean defaultt, boolean loadDescription) throws ParseException {
        
        this.nextVersion = null;
        this.linked = false;

        this.id = id;
        this.name = name;
        this.description = description;

        SimpleDateFormat sdf = new SimpleDateFormat(FORMAT_DATE);

        this.startDate = sdf.parse(startDate);
        this.endDate = sdf.parse(endDate);
        this.active = active;
        this.defaultFlag = defaultt;

        // Data Manager Prototype
        dataManager = new DataManager(buildRtBasePath(), loadDescription); // with descriptions

        // Claim Manager Prototype
        claimManager = new ClaimManager(buildClaimLayoutPath());
        LOG.log(Level.INFO, "Grouper version {0} instantiated", this.name);
    }

    private String buildRtBasePath() {
        String retPath = FileProperties.FILE_BASE + "/"
                + this.name.replaceAll("\\.", "") + "/RT";
        return retPath;
    }

    private String buildClaimLayoutPath() {
        String retPath = FileProperties.FILE_BASE + "/"
                + this.name.replaceAll("\\.", "");
        return retPath;
    }

    /**
     *
     * @param id
     * @param grouperVersionObject
     * @param loadDescription
     * @throws ParseException
     */
    public AbstractGrouper(Integer id, GrouperVersionObject grouperVersionObject,
            Boolean loadDescription) throws ParseException {
        this(id,
                grouperVersionObject.getId(),
                grouperVersionObject.getDescription(),
                grouperVersionObject.getStartDate(),
                grouperVersionObject.getEndDate(),
                Integer.parseInt(grouperVersionObject.getActiveFlag()) > 0,
                Integer.parseInt(grouperVersionObject.getDefaultFlag()) > 0,
                loadDescription
        );
    }

    /**
     * @return the id
     */
    @Override
    public int getId() {
        return id;
    }

    /**
     * @param id the id to set
     */
    @Override
    public void setId(int id) {
        this.id = id;
    }

    /**
     * @return the name
     */
    @Override
    public String getName() {
        return name;
    }

    /**
     * @param name the name to set
     */
    @Override
    public void setName(String name) {
        this.name = name;
    }

    /**
     * @return the description
     */
    @Override
    public String getDescription() {
        return description;
    }

    /**
     * @param description the description to set
     */
    @Override
    public void setDescription(String description) {
        this.description = description;
    }

    /**
     * @return the active
     */
    @Override
    public boolean isActive() {
        return active;
    }

    /**
     * @param active the active to set
     */
    public void setActive(boolean active) {
        this.active = active;
    }

    private int id;
    private String name;
    private String description;
    private boolean active;
    private Date startDate;
    private Date endDate;
    private boolean defaultFlag;

    /**
     * @return the defaultFlag
     */
    @Override
    public boolean isDefault() {
        return defaultFlag;
    }

    /**
     * @param defaultFlag the defaultFlag to set
     */
    public void setDefault(boolean defaultFlag) {
        this.defaultFlag = defaultFlag;
    }

    /**
     *
     * @return
     */
    @Override
    public ClaimContainerIF getEmptyClaim() {
        return new ClaimContainer();
    }

    /**
     * Convert String-based input claim object
     * into internal claim object
     * @param claimInput
     * @return 
     */
    protected ClaimIF castClaim(final ClaimContainerIF claimInput) {
        // convert to internal claim format
        ClaimAdapterIF adapter = new ClaimAdapter();
        ClaimIF claim = adapter.convert(claimInput);
        return claim;
    }
    
    /**
     *
     * @param claimInput
     */
    @Override
    public void group(final ClaimContainerIF claimInput) {
        // convert to internal format
        final ClaimIF claim = castClaim(claimInput);
        group(claim);

        // populate results back
        claimInput.setVersionUsed(claim.getGrouperVersion());
//        claimInput.setHippsCode(claim.getHippsCode().getHippsCode()); /* do not populate incomplete HIPPS */
        claimInput.setHippsCode(claim.getHippsCode().getFinalHippsCode());
        // populate HIPPS object (need to move that stuff to ClaimAdapter class
        HippsContainerIF hipps = new HippsContainer();
        if (claim.getHippsCode().isHippsComplete()) {
            for (int i = 1; i < hipps.getPositionsCount() + 1; i++) {
                String value = claim.getHippsCode().getValue(i).getValue();
                String posDesc = claim.getHippsCode().getValue(i).getPositionDescription();
                String valDesc = claim.getHippsCode().getValue(i).getValueDescription();
                hipps.setPositionValue(i, value);
                hipps.setPositionDescription(i, posDesc + ": " + valDesc);
            }
        }
        claimInput.setHippsContainer(hipps);
        // set validity
        claimInput.setValidityFlag(
                new ValidityFlagContainer(String.format("%02d",claim.getSingleValidityFlag().getId()),
                        claim.getSingleValidityFlag().getDescription())
        );
        // set return code
        claimInput.setReturnCode(
                new ReturnCodeContainer(String.format("%02d", claim.getSingleReturnCode().getId()),
                        claim.getSingleReturnCode().getDescription())
        );
        // set edits
        claimInput.setEdits(claim.getClaimEdits());
    }

    /**
     *
     * @param claim
     */
    protected void group(final ClaimIF claim) {

        final int GRC_INTERNAL_ERROR = 50; // initialization
        // process claim
        if (claim == null) { // no null claims allowed
            return;
        }

        try {
            processClaim(claim, getDataManager());
            // populate version
            claim.setGrouperVersion(getVersion());
        } catch (Exception e) {
            if(MetaData.EXTENDED_ERROR_MESSAGING) {
                e.printStackTrace();
            }
            claim.getReturnCodes().getCollection().clear();
            claim.getReturnCodes().getCollection().add(new ReturnCode(GRC_INTERNAL_ERROR, "Grouper Internal Error"));
            claim.setHippsCode(new HippsCode());
            claim.setGrouperVersion(getVersion());
            LOG.info("*** SEVERE: Grouper Internal Error with processing claim ***");
            // Logging for internal error
            for (EditIF edit : claim.getClaimEdits().getEdits()) {
                LOG.info(edit.toString());
            }
        }
    }

    /**
     *
     * @param claim
     * @param dataManager
     */
    protected void processClaim(final ClaimIF claim, final DataManagerIF dataManager) {
        HippsProcessor hp = new HippsProcessor();
        hp.process(claim, dataManager);
    }

    // Batch & MF grouping
    /**
     *
     * @param claimInput
     * @return
     */
    @Override
    public String group(String claimInput) {
        // read input claim
        ClaimObject claimIn = getClaimManager().loadClaim(claimInput);
        // convert to internal format
        final ClaimIF claim = castClaim(claimIn);
        // process claim
        group(claim);
        // format results
        String retStr = formatResult(claim);

        // populate back results
        // Version used
        claimIn.setVersionUsed(claim.getGrouperVersion());
        // HIPPS
        claimIn.setHippsCode(claim.getHippsCode().getHippsCode());
        // Validity flag
        claimIn.setByPosition(Claim_EN.VALIDITY_FLAG.getItemNumberStart(), String.format("%02d", claim.getSingleValidityFlag().getId()));
        // Return Code
        claimIn.setByPosition(Claim_EN.RETURN_CODE.getItemNumberStart(), String.format("%02d", claim.getSingleReturnCode().getId()));

        // Print logs
        String actionFlag = claimIn.getOptions().getActionFlag();
        if (actionFlag != null && actionFlag.equalsIgnoreCase(ActionFlag_EN.Y.name())) {
            LOG.info("*** Grouping Detail Report ***");
            LOG.info(claimIn.toString());
            LOG.log(Level.INFO, "*** Output string = {0}", retStr);
            claim.getClaimEdits().getEdits().forEach((edit) -> {
                LOG.info(edit.toString());
            });
        }

        return retStr;
    }
    
    /**
     *
     * @param claim
     * @return
     */
    protected String formatResult(final ClaimIF claim) {
        String retStr = getVersion()
                + claim.getHippsCode().getFinalHippsCode()
                + String.format("%02d", claim.getSingleValidityFlag().getId())
                + String.format("%02d", claim.getSingleReturnCode().getId());
        return retStr;
    }

    /**
     *
     * @return
     */
    @Override
    public String getVersion() {
        return getName();
    }

    /**
     *
     * @param tableType
     * @return
     */
    @Override
    public DataContainerIF<PositionAccessorIF> getAvailableTableContent(TableNames_EN tableType) {
        return getDataManager().getDataMap().get(tableType);
    }

    /**
     * @return the dataManager
     */
    public DataManagerIF getDataManager() {
        return dataManager;
    }

    /**
     * @param dataManager the dataManager to set
     */
    public void setDataManager(DataManagerIF dataManager) {
        this.dataManager = dataManager;
    }

    /**
     * @return the claimManager
     */
    @Override
    public ClaimManagerIF getClaimManager() {
        return claimManager;
    }

    /**
     * @param claimManager the claimManager to set
     */
    public void setClaimManager(ClaimManagerIF claimManager) {
        this.claimManager = claimManager;
    }
}
